"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: D410, E501, E101, D206
from apimatic_core.authentication.multiple.single_auth import (
    Single,
)
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter

from mindbodypublicapi.api_helper import APIHelper
from mindbodypublicapi.configuration import (
    Server,
)
from mindbodypublicapi.controllers.base_controller import (
    BaseController,
)
from mindbodypublicapi.http.http_method_enum import (
    HttpMethodEnum,
)
from mindbodypublicapi.models.checkout_shopping_cart_response import (
    CheckoutShoppingCartResponse,
)
from mindbodypublicapi.models.get_alternative_payment_methods_response import (
    GetAlternativePaymentMethodsResponse,
)
from mindbodypublicapi.models.get_contracts_response import (
    GetContractsResponse,
)
from mindbodypublicapi.models.get_custom_payment_methods_response import (
    GetCustomPaymentMethodsResponse,
)
from mindbodypublicapi.models.get_gift_card_balance_response import (
    GetGiftCardBalanceResponse,
)
from mindbodypublicapi.models.get_gift_card_response import (
    GetGiftCardResponse,
)
from mindbodypublicapi.models.get_packages_response import (
    GetPackagesResponse,
)
from mindbodypublicapi.models.get_products_inventory_response import (
    GetProductsInventoryResponse,
)
from mindbodypublicapi.models.get_products_response import (
    GetProductsResponse,
)
from mindbodypublicapi.models.get_sales_response import (
    GetSalesResponse,
)
from mindbodypublicapi.models.get_services_response import (
    GetServicesResponse,
)
from mindbodypublicapi.models.get_transactions_response import (
    GetTransactionsResponse,
)
from mindbodypublicapi.models.purchase_account_credit_response import (
    PurchaseAccountCreditResponse,
)
from mindbodypublicapi.models.purchase_contract_response import (
    PurchaseContractResponse,
)
from mindbodypublicapi.models.purchase_gift_card_response import (
    PurchaseGiftCardResponse,
)
from mindbodypublicapi.models.return_sale_response import (
    ReturnSaleResponse,
)
from mindbodypublicapi.models.update_product_price_response import (
    UpdateProductPriceResponse,
)
from mindbodypublicapi.models.update_sale_date_response import (
    UpdateSaleDateResponse,
)
from mindbodypublicapi.models.update_service_response import (
    UpdateServiceResponse,
)


class SaleController(BaseController):
    """A Controller to access Endpoints in the mindbodypublicapi API."""

    def __init__(self, config):
        """Initialize SaleController object."""
        super(SaleController, self).__init__(config)

    def get_accepted_card_types(self,
                                version,
                                site_id,
                                authorization=None):
        """Perform a GET request to /public/v{version}/sale/acceptedcardtypes.

        Gets a list of card types that the site accepts. You can also use `GET Sites`
        to return the Site object, which contains individual accepted card types for
        requested sites.
        This endpoint has no query parameters.The response returns a list of strings.
        Possible values are:
        * Visa
        * MasterCard
        * Discover
        * AMEX

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            List[str]: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/acceptedcardtypes")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize),
        ).execute()

    def get_alternative_payment_methods(self,
                                        version,
                                        site_id,
                                        authorization=None,
                                        request_client_id=None,
                                        request_location_id=None):
        """Perform a GET request to
        /public/v{version}/sale/alternativepaymentmethods.

        Get alternative and local payment methods that are allowed for a site.
        These payment methods can later be used in `Initiate Checkout Shopping Cart
        Using Alternative Payments`
        and `Initiate Purchase Contract Using Alternative Payments` to make the
        payment.

        The currently supported alternative payments are:
        - iDEAL
        - Apple Pay
        **Notes:**
        - This endpoint is only available for Studios on MBPS (Mindbody Payments)
        with Stripe payment processor.
        - This endpoint only supports the online store location (LocationId = 98). If
        LocationId is not provided, it will default to 98.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_client_id (str, optional): The client ID
            request_location_id (int, optional): The location ID  Default: **null**
                (Online store location)

        Returns:
            GetAlternativePaymentMethodsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/alternativepaymentmethods")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.clientId")
                .value(request_client_id))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetAlternativePaymentMethodsResponse.from_dictionary),
        ).execute()

    def get_contracts(self,
                      version,
                      request_location_id,
                      site_id,
                      authorization=None,
                      request_contract_ids=None,
                      request_limit=None,
                      request_offset=None,
                      request_promo_code=None,
                      request_sold_online=None,
                      request_unique_client_id=None):
        """Perform a GET request to /public/v{version}/sale/contracts.

        Returns the contracts and autopay options that are available on a
        location-by-location basis. Depending on the configurations established by
        the site, this endpoint returns options that can be used to sign up clients
        for recurring payments for services offered by the business.

        Args:
            version (str): version of the api.
            request_location_id (int): The ID of the location that has the requested
                contracts and AutoPay options.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_contract_ids (List[int], optional): When included, the response
                only contains details about the specified contract IDs.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_offset (int, optional): Page offset, defaults to 0.
            request_promo_code (str, optional): PromoCode to apply
            request_sold_online (bool, optional): When `true`, the response only
                contains details about contracts and AutoPay options that can be sold
                online.   When `false`, all contracts are returned.  Default:
                **false**
            request_unique_client_id (int, optional): The ID of the client.

        Returns:
            GetContractsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/contracts")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.contractIds")
                .value(request_contract_ids))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.promoCode")
                .value(request_promo_code))
            .query_param(Parameter()
                .key("request.soldOnline")
                .value(request_sold_online))
            .query_param(Parameter()
                .key("request.uniqueClientId")
                .value(request_unique_client_id))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetContractsResponse.from_dictionary),
        ).execute()

    def get_custom_payment_methods(self,
                                   version,
                                   site_id,
                                   authorization=None,
                                   request_limit=None,
                                   request_offset=None):
        """Perform a GET request to
        /public/v{version}/sale/custompaymentmethods.

        Get payment methods that can be used to pay for sales at a site.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_offset (int, optional): Page offset, defaults to 0.

        Returns:
            GetCustomPaymentMethodsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/custompaymentmethods")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetCustomPaymentMethodsResponse.from_dictionary),
        ).execute()

    def get_gift_card_balance(self,
                              version,
                              site_id,
                              authorization=None,
                              barcode_id=None):
        """Perform a GET request to /public/v{version}/sale/giftcardbalance.

        Returns a gift card’s remaining balance.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            barcode_id (str, optional): The barcode ID of the gift card for which you
                want the balance.

        Returns:
            GetGiftCardBalanceResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/giftcardbalance")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("barcodeId")
                .value(barcode_id))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetGiftCardBalanceResponse.from_dictionary),
        ).execute()

    def get_gift_cards(self,
                       version,
                       site_id,
                       authorization=None,
                       request_ids=None,
                       request_include_custom_layouts=None,
                       request_limit=None,
                       request_location_id=None,
                       request_offset=None,
                       request_sold_online=None):
        """Perform a GET request to /public/v{version}/sale/giftcards.

        Returns information about gift cards that can be purchased.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_ids (List[int], optional): Filters the results to the requested
                gift card IDs.<br />  Default: **all** gift cards.
            request_include_custom_layouts (bool, optional): When `true`, includes
                custom gift card layouts.<br />  When `false`, includes only system
                layouts.  Default: **false**
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): When included, returns gift cards
                that are sold at the provided location ID.
            request_offset (int, optional): Page offset, defaults to 0.
            request_sold_online (bool, optional): When `true`, only returns gift
                cards that are sold online.<br />  Default: **false**

        Returns:
            GetGiftCardResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/giftcards")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.ids")
                .value(request_ids))
            .query_param(Parameter()
                .key("request.includeCustomLayouts")
                .value(request_include_custom_layouts))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.soldOnline")
                .value(request_sold_online))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetGiftCardResponse.from_dictionary),
        ).execute()

    def get_packages(self,
                     version,
                     site_id,
                     authorization=None,
                     request_limit=None,
                     request_location_id=None,
                     request_offset=None,
                     request_package_ids=None,
                     request_sell_online=None):
        """Perform a GET request to /public/v{version}/sale/packages.

        A package is typically used to combine multiple services and/or products into
        a single offering. Staff members can check out multiple appointments while
        selling the package, and can discount the items included. For example, a spa
        might bundle a massage, a pedicure, a manicure, a facial, and a few selected
        beauty products into a package.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): The location ID to use to determine
                the tax for the products that this request returns.<br />  Default:
                **online store**
            request_offset (int, optional): Page offset, defaults to 0.
            request_package_ids (List[int], optional): A list of the packages IDs to
                filter by.
            request_sell_online (bool, optional): When `true`, only returns products
                that can be sold online.<br />  When `false`, all products are
                returned.<br />  Default: **false**

        Returns:
            GetPackagesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/packages")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.packageIds")
                .value(request_package_ids))
            .query_param(Parameter()
                .key("request.sellOnline")
                .value(request_sell_online))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetPackagesResponse.from_dictionary),
        ).execute()

    def get_products(self,
                     version,
                     site_id,
                     authorization=None,
                     request_category_ids=None,
                     request_limit=None,
                     request_location_id=None,
                     request_offset=None,
                     request_product_ids=None,
                     request_search_text=None,
                     request_secondary_category_ids=None,
                     request_sell_online=None,
                     request_sub_category_ids=None):
        """Perform a GET request to /public/v{version}/sale/products.

        Get retail products available for purchase at a site.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_category_ids (List[int], optional): A list of revenue category
                IDs to filter by. Use this ID when calling the GET Categories
                endpoint.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): The location ID to use to determine
                the tax for the products that this request returns.<br />  Default:
                **online store**
            request_offset (int, optional): Page offset, defaults to 0.
            request_product_ids (List[str], optional): The barcode number of the
                product to be filter by.
            request_search_text (str, optional): A search filter, used for searching
                by term.
            request_secondary_category_ids (List[int], optional): A list of secondary
                categories to filter by. Use this ID when calling the GET Categories
                endpoint.
            request_sell_online (bool, optional): When `true`, only products that can
                be sold online are returned.<br />  When `false`, all products are
                returned.<br />  Default: **false**
            request_sub_category_ids (List[int], optional): A list of subcategory IDs
                to filter by. Use this ID when calling the GET Categories endpoint.

        Returns:
            GetProductsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/products")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.categoryIds")
                .value(request_category_ids))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.productIds")
                .value(request_product_ids))
            .query_param(Parameter()
                .key("request.searchText")
                .value(request_search_text))
            .query_param(Parameter()
                .key("request.secondaryCategoryIds")
                .value(request_secondary_category_ids))
            .query_param(Parameter()
                .key("request.sellOnline")
                .value(request_sell_online))
            .query_param(Parameter()
                .key("request.subCategoryIds")
                .value(request_sub_category_ids))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetProductsResponse.from_dictionary),
        ).execute()

    def update_products(self,
                        version,
                        site_id,
                        update_products_requests,
                        authorization=None):
        """Perform a PUT request to /public/v{version}/sale/products.

        Update retail products available for purchase at a site.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            update_products_requests (List[UpdateProductRequest]): The request body
                parameter.
            authorization (str, optional): A staff user authorization token.

        Returns:
            GetProductsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/products")
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .body_param(Parameter()
                .value(update_products_requests))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetProductsResponse.from_dictionary),
        ).execute()

    def get_products_inventory(self,
                               version,
                               site_id,
                               authorization=None,
                               request_barcode_ids=None,
                               request_limit=None,
                               request_location_ids=None,
                               request_offset=None,
                               request_product_ids=None):
        """Perform a GET request to /public/v{version}/sale/productsinventory.

        Get retail products inventory data available at a site.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_barcode_ids (List[str], optional): When included, the response
                only contains details about the specified Barcode Ids.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_ids (List[int], optional): When included, the response
                only contains details about the specified location Ids.
            request_offset (int, optional): Page offset, defaults to 0.
            request_product_ids (List[str], optional): When included, the response
                only contains details about the specified product Ids.

        Returns:
            GetProductsInventoryResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/productsinventory")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.barcodeIds")
                .value(request_barcode_ids))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationIds")
                .value(request_location_ids))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.productIds")
                .value(request_product_ids))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetProductsInventoryResponse.from_dictionary),
        ).execute()

    def get_purchase_contract_status(self,
                                     version,
                                     access_token,
                                     site_id,
                                     authorization=None):
        """Perform a GET request to
        /public/v{version}/sale/purchasecontractstatus.

        Fetches the status of an initiate purchase contract given accessToken.
        This endpoint complements the `Initiate Purchase Contract Using Alternative
        Payments` endpoint. `Initiate Purchase Contract Using Alternative Payments`
        endpoint will capture the intent to purchase a contract, and return a
        redirect URL for the end user to make the payment. Once the payment
        concludes, the end user will be redirected back to the
        PaymentAuthenticationCallbackUrl, which was provided while invoking the
        `Initiate Purchase Contract Using Alternative Payments` endpoint. You can
        then invoke this endpoint to obtain the `PurchaseContractResponse`.

        **For a comprehensive guide, follow this tutorial:** [Purchase a
        Contract](https://developers.mindbodyonline.com/ui/documentation/public-api#/h
        ttp/tutorials/purchase-a-contract)

        The currently supported alternative payments are:
        - iDEAL
        - Apple Pay

        Args:
            version (str): version of the api.
            access_token (str): The request query parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            PurchaseContractResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/purchasecontractstatus")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .query_param(Parameter()
                .key("accessToken")
                .value(access_token))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PurchaseContractResponse.from_dictionary),
        ).execute()

    def get_sales(self,
                  version,
                  site_id,
                  authorization=None,
                  request_end_sale_date_time=None,
                  request_limit=None,
                  request_offset=None,
                  request_payment_method_id=None,
                  request_sale_id=None,
                  request_start_sale_date_time=None):
        """Perform a GET request to /public/v{version}/sale/sales.

        Get sales completed at a site.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_end_sale_date_time (datetime, optional): Filters results to sales
                that happened before this date and time.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_offset (int, optional): Page offset, defaults to 0.
            request_payment_method_id (int, optional): Filters results to sales paid
                for by the given payment method ID which indicates payment method(s)
                (i.e. cash, VISA, AMEX, Check, etc.).
            request_sale_id (int, optional): The sale ID associated with the
                particular item. It Filters results to the requested sale ID.
            request_start_sale_date_time (datetime, optional): Filters results to
                sales that happened after this date and time.

        Returns:
            GetSalesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/sales")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.endSaleDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_end_sale_date_time)))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.paymentMethodId")
                .value(request_payment_method_id))
            .query_param(Parameter()
                .key("request.saleId")
                .value(request_sale_id))
            .query_param(Parameter()
                .key("request.startSaleDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_start_sale_date_time)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetSalesResponse.from_dictionary),
        ).execute()

    def get_services(self,
                     version,
                     site_id,
                     authorization=None,
                     request_class_id=None,
                     request_class_schedule_id=None,
                     request_hide_related_programs=None,
                     request_include_discontinued=None,
                     request_include_sale_in_contract_only=None,
                     request_limit=None,
                     request_location_id=None,
                     request_offset=None,
                     request_program_ids=None,
                     request_sell_online=None,
                     request_service_ids=None,
                     request_session_type_ids=None,
                     request_staff_id=None):
        """Perform a GET request to /public/v{version}/sale/services.

        Get pricing options available for purchase at a site

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_class_id (int, optional): Filters to the pricing options for the
                specified class ID.
            request_class_schedule_id (int, optional): Filters to the pricing options
                for the specified class schedule ID.
            request_hide_related_programs (bool, optional): When `true`, indicates
                that pricing options of related programs are omitted from the
                response.<br />  Default: **false**
            request_include_discontinued (bool, optional): When `true`, indicates
                that the filtered pricing option list includes discontinued pricing
                options.<br />  Default: **false**
            request_include_sale_in_contract_only (bool, optional): When `true`,
                indicates that the filtered pricing option list includes sale in
                contract only pricing options.<br />  Default: **false**
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): When specified, for each returned
                pricing option, `TaxRate` and `TaxIncluded` are calculated according
                to the specified location. Note that this does not filter results to
                only services provided at the given location, and for locations where
                Value-Added Tax (VAT) rules apply, the `TaxRate` is set to zero.
            request_offset (int, optional): Page offset, defaults to 0.
            request_program_ids (List[int], optional): Filters to pricing options
                with the specified program IDs.
            request_sell_online (bool, optional): When `true`, filters the pricing
                options to display only those available for online purchase.  This
                parameter is only applicable in Business Mode (when a staff
                authentication header is included) and ignored in Consumer Mode (when
                no authentication header is passed).  <br />Default: `false` (for
                staff users)  <br />Business Mode: This parameter controls the
                filtering behavior. Staff users can set this to `true`  to show only
                pricing options that can be sold online, or `false` to show all
                available pricing options.  <br />Consumer Mode: This value is
                automatically set to `true` and cannot be overridden,  ensuring
                consumers only see pricing options available for online purchase.
            request_service_ids (List[str], optional): Filters to the pricing options
                with the specified IDs. In this context, service and pricing option
                are used interchangeably. These are the `PurchasedItems[].Id`
                returned from GET Sales.
            request_session_type_ids (List[int], optional): Filters to the pricing
                options with the specified session types IDs.
            request_staff_id (int, optional): Sets `Price` and `OnlinePrice` to the
                particular pricing of a specific staff member, if allowed by the
                business.

        Returns:
            GetServicesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/services")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.classId")
                .value(request_class_id))
            .query_param(Parameter()
                .key("request.classScheduleId")
                .value(request_class_schedule_id))
            .query_param(Parameter()
                .key("request.hideRelatedPrograms")
                .value(request_hide_related_programs))
            .query_param(Parameter()
                .key("request.includeDiscontinued")
                .value(request_include_discontinued))
            .query_param(Parameter()
                .key("request.includeSaleInContractOnly")
                .value(request_include_sale_in_contract_only))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.programIds")
                .value(request_program_ids))
            .query_param(Parameter()
                .key("request.sellOnline")
                .value(request_sell_online))
            .query_param(Parameter()
                .key("request.serviceIds")
                .value(request_service_ids))
            .query_param(Parameter()
                .key("request.sessionTypeIds")
                .value(request_session_type_ids))
            .query_param(Parameter()
                .key("request.staffId")
                .value(request_staff_id))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetServicesResponse.from_dictionary),
        ).execute()

    def update_services(self,
                        version,
                        site_id,
                        update_services_request,
                        authorization=None):
        """Perform a PUT request to /public/v{version}/sale/services.

        Update unit price and online price of provided services.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            update_services_request (List[UpdateServiceRequest]): The request body
                parameter.
            authorization (str, optional): A staff user authorization token.

        Returns:
            UpdateServiceResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/services")
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .body_param(Parameter()
                .value(update_services_request))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UpdateServiceResponse.from_dictionary),
        ).execute()

    def get_transactions(self,
                         version,
                         site_id,
                         authorization=None,
                         request_client_id=None,
                         request_limit=None,
                         request_location_id=None,
                         request_offset=None,
                         request_sale_id=None,
                         request_status=None,
                         request_transaction_end_date_time=None,
                         request_transaction_id=None,
                         request_transaction_start_date_time=None):
        """Perform a GET request to /public/v{version}/sale/transactions.

        This endpoint returns a list of transaction details of processed sales.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_client_id (int, optional): Filters results to the requested
                client ID.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): Filters the transaction results with
                the ID number associated with the location of the sale.
            request_offset (int, optional): Page offset, defaults to 0.
            request_sale_id (int, optional): Filters the transaction results with the
                ID number associated with the sale.
            request_status (str, optional): Filters the transaction results by the
                estimated transaction status.
            request_transaction_end_date_time (datetime, optional): Filters the
                transaction results that happpened before this date and time.
                Default: **today’s date**
            request_transaction_id (int, optional): Filters the transaction results
                with the ID number generated when the sale is processed.
            request_transaction_start_date_time (datetime, optional): Filters the
                transaction results that happpened after this date and time.
                Default: **today’s date**

        Returns:
            GetTransactionsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/transactions")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.clientId")
                .value(request_client_id))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.saleId")
                .value(request_sale_id))
            .query_param(Parameter()
                .key("request.status")
                .value(request_status))
            .query_param(Parameter()
                .key("request.transactionEndDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_transaction_end_date_time)))
            .query_param(Parameter()
                .key("request.transactionId")
                .value(request_transaction_id))
            .query_param(Parameter()
                .key("request.transactionStartDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_transaction_start_date_time)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetTransactionsResponse.from_dictionary),
        ).execute()

    def checkout_shopping_cart(self,
                               version,
                               request,
                               site_id,
                               authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/checkoutshoppingcart.

        This endpoint provides a wide range of functionality. For example, you can
        use it when a client purchases new pricing options, retail products,
        packages, and tips. You can also combine purchasing a new pricing option and
        many other functions, such as booking a client into a class, booking a new
        appointment for a client, enrolling a client into an enrollment or course, or
        reconciling an unpaid, already booked appointment or class. Use this call
        when a client purchases:
        * a pricing option, after calling `GET Services` and choosing a specific
        pricing option’s ID
        * a retail product, after calling `GET Products` and choosing a specific
        retail product’s ID
        * a package, after calling `GET Packages` and choosing a specific package’s ID
        * a tip to give to a staff member, after calling `GET Staff` and choosing a
        specific staff member ID, and the amount that the client wants to tip
        The documentation provides explanations of the request body and response, as
        well as the cart item metadata, payment item metadata, and purchased cart
        items.
        This endpoint had been updated to support Strong Customer Authentication
        (SCA).
        **Note :**
        Protect yourself from processor fees and credit card fraud.Remember to always
        protect your web forms that leverage POST CheckoutShoppingCart, POST
        PurchaseContract or POST PurchaseGiftCard with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (CheckoutShoppingCartRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            CheckoutShoppingCartResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/checkoutshoppingcart")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CheckoutShoppingCartResponse.from_dictionary),
        ).execute()

    def complete_checkout_shopping_cart_using_alternative_payments(self,
                                                                   version,
                                                                   request,
                                                                   site_id,
                                                                   authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/completecheckoutshoppingcart.

        This endpoint complements the InitiateCheckoutShoppintCart endpoint. Please
        visit its documentation to understand further.
        The currently supported alternative payments are:
        - iDEAL
        - Apple Pay
        **How to use :**
        InitiateCheckoutShoppingCart and CompleteCheckoutShoppingCart endpoints work
        together. InitiateCheckoutShoppingCart endpoint will capture the intent to
        perform a checkout, and return a redirect URL for the end user to make the
        payment. Once the payment concludes, the end user will be redirected back to
        the PaymentAuthenticationCallbackUrl, which was provided while invoking the
        InitiateCheckoutShoppingCart endpoint. You can then invoke the
        CompleteCheckoutShoppingCart endpoint to complete the remaining checkout
        activities and obtain the CheckoutShoppingCartResponse.
        **For a comprehensive guide, follow this tutorial:** [Checkout Shopping Cart
        Using Alternative
        Payments](https://developers.mindbodyonline.com/ui/documentation/public-api#/h
        ttp/tutorials/checkout-shopping-cart-using-alternative-payments)
        **Notes:**
        - This endpoint is only available for Studios on MBPS (Mindbody Payments)
        with Stripe payment processor.
        - Protect yourself from processor fees. Remember to always protect your web
        forms that leverage POST InitiateCheckoutShoppingCart or POST
        CompleteCheckoutShoppingCart with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (CompleteCheckoutShoppingCartUsingAlternativePaymentsRequest):
                The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            CheckoutShoppingCartResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/completecheckoutshoppingcart")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CheckoutShoppingCartResponse.from_dictionary),
        ).execute()

    def initiate_checkout_shopping_cart_using_alternative_payments(self,
                                                                   version,
                                                                   request,
                                                                   site_id,
                                                                   authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/initiatecheckoutshoppingcart.

        This endpoint provides a wide range of functionality. For example, you can
        use it when a client purchases new pricing options, retail products,
        packages, and tips. You can also combine purchasing a new pricing option and
        many other functions, such as booking a client into a class, booking a new
        appointment for a client, enrolling a client into an enrollment or course, or
        reconciling an unpaid, already booked appointment or class. Use this call
        when a client purchases:
        * a pricing option, after calling `GET Services` and choosing a specific
        pricing option’s ID
        * a retail product, after calling `GET Products` and choosing a specific
        retail product’s ID
        * a package, after calling `GET Packages` and choosing a specific package’s ID
        * a tip to give to a staff member, after calling `GET Staff` and choosing a
        specific staff member ID, and the amount that the client wants to tip
        The documentation provides explanations of the request body and response, as
        well as the cart item metadata, payment item metadata, and purchased cart
        items.
        The currently supported alternative payments are:
        - iDEAL
        - Apple Pay

        **How to use :**
        InitiateCheckoutShoppingCart and CompleteCheckoutShoppingCart endpoints work
        together. InitiateCheckoutShoppingCart endpoint will capture the intent to
        perform a checkout, and return a redirect URL for the end user to make the
        payment. Once the payment concludes, the end user will be redirected back to
        the PaymentAuthenticationCallbackUrl, which was provided while invoking the
        InitiateCheckoutShoppingCart endpoint. You can then invoke the
        CompleteCheckoutShoppingCart endpoint to complete the remaining checkout
        activities and obtain the CheckoutShoppingCartResponse.
        **For a comprehensive guide, follow this tutorial:** [Checkout Shopping Cart
        Using Alternative
        Payments](https://developers.mindbodyonline.com/ui/documentation/public-api#/h
        ttp/tutorials/checkout-shopping-cart-using-alternative-payments)
        **Notes:**
        - This endpoint is only available for Studios on MBPS (Mindbody Payments)
        with Stripe payment processor.
        - This endpoint only supports the online store location (LocationId = 98). If
        LocationId is not provided, it will default to 98.
        - Product purchases are currently not supported on the online store location.
        - This endpoint only supports those payment methods as obtained from the
        GetAlternativePaymentMethods endpoint.
        - Protect yourself from processor fees. Remember to always protect your web
        forms that leverage POST InitiateCheckoutShoppingCart or POST
        CompleteCheckoutShoppingCart with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (InitiateCheckoutShoppingCartUsingAlternativePaymentsRequest):
                The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            Any: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/initiatecheckoutshoppingcart")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize),
        ).execute()

    def initiate_purchase_contract_using_alternative_payments(self,
                                                              version,
                                                              request,
                                                              site_id,
                                                              authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/initiatepurchasecontract.

        Allows a client to sign up for a contract or autopay using the information
        returned from the `GET Contracts` endpoint. The client can pay with allowed
        alternative and local payment methods returned from `Get Alternative Payment
        Method`. The client must exist at the site specified before this call is made.

        The currently supported alternative payments are:
        - iDEAL
        - Apple Pay

        **For a comprehensive guide, follow this tutorial:** [Purchase a
        Contract](https://developers.mindbodyonline.com/ui/documentation/public-api#/h
        ttp/tutorials/purchase-a-contract)

        **Notes:**
        - This endpoint is only available for Studios on MBPS (Mindbody Payments)
        with Stripe payment processor.
        - This endpoint only supports the online store location (LocationId = 98). If
        LocationId is not provided, it will default to 98.
        - This endpoint only supports those payment methods as obtained from the
        GetAlternativePaymentMethods endpoint.
        - Protect yourself from processor fees and credit card fraud. Remember to
        always protect your web forms that leverage POST CheckoutShoppingCart, POST
        PurchaseContract or POST PurchaseGiftCard with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (InitiatePurchaseContractRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            Any: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/initiatepurchasecontract")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize),
        ).execute()

    def purchase_account_credit(self,
                                version,
                                request,
                                site_id,
                                authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/purchaseaccountcredit.

        Allows a client to purchase account credit from a business.

        Args:
            version (str): version of the api.
            request (PurchaseAccountCreditRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            PurchaseAccountCreditResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/purchaseaccountcredit")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PurchaseAccountCreditResponse.from_dictionary),
        ).execute()

    def purchase_contract(self,
                          version,
                          request,
                          site_id,
                          authorization=None):
        """Perform a POST request to /public/v{version}/sale/purchasecontract.

        Allows a client to sign up for a contract or autopay using the information
        returned from the `GET Contracts` endpoint. The client can pay with a new
        credit card or with a stored credit card. The client must exist at the site
        specified before this call is made.
        This endpoint allows a developer to specify whether a client pays now or pays
        on the `StartDate`. If you are building a client-facing experience, you
        should talk with the business owner to understand the owner’s policies before
        you give clients a choice of the two payment types.
        This endpoint also allows a developer to specify a `ProrateDate`. If the date
        is passed, the Totals returned will always include the pro-rate amount for
        instant payment, like:
        - `FirstPaymentOccurs` = `Instant` =&gt; returns instant payment total for
        the contract + pro-rate amount.
        - `FirstPaymentOccurs` = `StartDate` =&gt; returns pro-rate amount + contract
        amount requiring instant payment. The rest of the contract will be due on
        `StartDate`.
        - Pro-rate amount payment on `StartDate` is not supported by this endpoint.
        This endpoint has been updated to support Strong Customer Authentication
        (SCA).
        **Note:**
        Protect yourself from processor fees and credit card fraud. Remember to
        always protect your web forms that leverage POST CheckoutShoppingCart, POST
        PurchaseContract or POST PurchaseGiftCard with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (PurchaseContractRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            PurchaseContractResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/purchasecontract")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PurchaseContractResponse.from_dictionary),
        ).execute()

    def purchase_gift_card(self,
                           version,
                           request,
                           site_id,
                           authorization=None):
        """Perform a POST request to /public/v{version}/sale/purchasegiftcard.

         Allows a client to purchase a gift card from a business in a variety of
        designs. The card can be emailed to the recipient on a specific day, and a
        card title and a personal message can be added.
         **Note**
        Protect yourself from processor fees and credit card fraud.Remember to always
        protect your web forms that leverage POST CheckoutShoppingCart, POST
        PurchaseContract or POST PurchaseGiftCard with a CAPTCHA!

        Args:
            version (str): version of the api.
            request (PurchaseGiftCardRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            PurchaseGiftCardResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/purchasegiftcard")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PurchaseGiftCardResponse.from_dictionary),
        ).execute()

    def return_sale(self,
                    version,
                    return_sale_request,
                    site_id,
                    authorization=None):
        """Perform a POST request to /public/v{version}/sale/returnsale.

        Return a comped sale for a specified sale ID in business mode. The sale is
        returnable only if it is a sale of a service, product or gift card and it has
        not been used. Currently, only the comp payment method is supported.

        Args:
            version (str): version of the api.
            return_sale_request (ReturnSaleRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            ReturnSaleResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/returnsale")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(return_sale_request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ReturnSaleResponse.from_dictionary),
        ).execute()

    def update_product_price(self,
                             version,
                             request,
                             site_id,
                             authorization=None):
        """Perform a POST request to
        /public/v{version}/sale/updateproductprice.

        This endpoint updates the retail price and an online price for a product.
        Passing at least one of them is mandatory.

        Args:
            version (str): version of the api.
            request (UpdateProductPriceRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            UpdateProductPriceResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/updateproductprice")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UpdateProductPriceResponse.from_dictionary),
        ).execute()

    def update_sale_date(self,
                         version,
                         request,
                         site_id,
                         authorization=None):
        """Perform a PUT request to /public/v{version}/sale/updatesaledate.

        This endpoint updates the SaleDate and returns the details of the sale.

        Args:
            version (str): version of the api.
            request (UpdateSaleDateRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            UpdateSaleDateResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/sale/updatesaledate")
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(UpdateSaleDateResponse.from_dictionary),
        ).execute()



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.api_helper import APIHelper
from mindbodypublicapi.models.client_index_value import (
    ClientIndexValue,
)


class ClientIndex(object):
    """Implementation of the 'ClientIndex' model.

    A client index.

    Attributes:
        id (int): The unique ID of the client index.
        name (str): The name of the client index.
        required_business_mode (bool): When `true`, indicates that the index is
            required when creating profiles in business mode.
        required_consumer_mode (bool): When `true`, indicates that the index is
            required when creating profiles in consumer mode.
        values (List[ClientIndexValue]): Contains a list with a single object
            representing the index value assigned to the client index.
        action (Action8Enum): The action performed on this object.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": "Id",
        "name": "Name",
        "required_business_mode": "RequiredBusinessMode",
        "required_consumer_mode": "RequiredConsumerMode",
        "values": "Values",
        "action": "Action",
    }

    _optionals = [
        "id",
        "name",
        "required_business_mode",
        "required_consumer_mode",
        "values",
        "action",
    ]

    def __init__(
        self,
        id=APIHelper.SKIP,
        name=APIHelper.SKIP,
        required_business_mode=APIHelper.SKIP,
        required_consumer_mode=APIHelper.SKIP,
        values=APIHelper.SKIP,
        action=APIHelper.SKIP):
        """Initialize a ClientIndex instance."""
        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id
        if name is not APIHelper.SKIP:
            self.name = name
        if required_business_mode is not APIHelper.SKIP:
            self.required_business_mode = required_business_mode
        if required_consumer_mode is not APIHelper.SKIP:
            self.required_consumer_mode = required_consumer_mode
        if values is not APIHelper.SKIP:
            self.values = values
        if action is not APIHelper.SKIP:
            self.action = action

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        id =\
            dictionary.get("Id")\
            if dictionary.get("Id")\
                else APIHelper.SKIP
        name =\
            dictionary.get("Name")\
            if dictionary.get("Name")\
                else APIHelper.SKIP
        required_business_mode =\
            dictionary.get("RequiredBusinessMode")\
            if "RequiredBusinessMode" in dictionary.keys()\
                else APIHelper.SKIP
        required_consumer_mode =\
            dictionary.get("RequiredConsumerMode")\
            if "RequiredConsumerMode" in dictionary.keys()\
                else APIHelper.SKIP
        values = None
        if dictionary.get("Values") is not None:
            values = [
                ClientIndexValue.from_dictionary(x)
                    for x in dictionary.get("Values")
            ]
        else:
            values = APIHelper.SKIP
        action =\
            dictionary.get("Action")\
            if dictionary.get("Action")\
                else APIHelper.SKIP

        # Return an object of this model
        return cls(id,
                   name,
                   required_business_mode,
                   required_consumer_mode,
                   values,
                   action)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _id=(
            self.id
            if hasattr(self, "id")
            else None
        )
        _name=(
            self.name
            if hasattr(self, "name")
            else None
        )
        _required_business_mode=(
            self.required_business_mode
            if hasattr(self, "required_business_mode")
            else None
        )
        _required_consumer_mode=(
            self.required_consumer_mode
            if hasattr(self, "required_consumer_mode")
            else None
        )
        _values=(
            self.values
            if hasattr(self, "values")
            else None
        )
        _action=(
            self.action
            if hasattr(self, "action")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"id={_id!r}, "
            f"name={_name!r}, "
            f"required_business_mode={_required_business_mode!r}, "
            f"required_consumer_mode={_required_consumer_mode!r}, "
            f"values={_values!r}, "
            f"action={_action!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _id=(
            self.id
            if hasattr(self, "id")
            else None
        )
        _name=(
            self.name
            if hasattr(self, "name")
            else None
        )
        _required_business_mode=(
            self.required_business_mode
            if hasattr(self, "required_business_mode")
            else None
        )
        _required_consumer_mode=(
            self.required_consumer_mode
            if hasattr(self, "required_consumer_mode")
            else None
        )
        _values=(
            self.values
            if hasattr(self, "values")
            else None
        )
        _action=(
            self.action
            if hasattr(self, "action")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"id={_id!s}, "
            f"name={_name!s}, "
            f"required_business_mode={_required_business_mode!s}, "
            f"required_consumer_mode={_required_consumer_mode!s}, "
            f"values={_values!s}, "
            f"action={_action!s}, "
            f")"
        )



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.api_helper import APIHelper
from mindbodypublicapi.models.client_arrival import (
    ClientArrival,
)
from mindbodypublicapi.models.client_contract import (
    ClientContract,
)
from mindbodypublicapi.models.client_membership import (
    ClientMembership,
)
from mindbodypublicapi.models.client_service import (
    ClientService,
)
from mindbodypublicapi.models.client_with_suspension_info import (
    ClientWithSuspensionInfo,
)


class GetClientCompleteInfoResponse(object):
    """Implementation of the 'GetClientCompleteInfoResponse' model.

    Contains information about the requested client.

    Attributes:
        client (ClientWithSuspensionInfo): A Client DTO with Suspension Informatoin
        client_services (List[ClientService]): Contains information about client
            pricing options.
        client_contracts (List[ClientContract]): Contains information about client
            contract.
        client_memberships (List[ClientMembership]): Contains information about
            client Memberships.
        client_arrivals (List[ClientArrival]): Contains information about client
            arrival services.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "client": "Client",
        "client_services": "ClientServices",
        "client_contracts": "ClientContracts",
        "client_memberships": "ClientMemberships",
        "client_arrivals": "ClientArrivals",
    }

    _optionals = [
        "client",
        "client_services",
        "client_contracts",
        "client_memberships",
        "client_arrivals",
    ]

    def __init__(
        self,
        client=APIHelper.SKIP,
        client_services=APIHelper.SKIP,
        client_contracts=APIHelper.SKIP,
        client_memberships=APIHelper.SKIP,
        client_arrivals=APIHelper.SKIP):
        """Initialize a GetClientCompleteInfoResponse instance."""
        # Initialize members of the class
        if client is not APIHelper.SKIP:
            self.client = client
        if client_services is not APIHelper.SKIP:
            self.client_services = client_services
        if client_contracts is not APIHelper.SKIP:
            self.client_contracts = client_contracts
        if client_memberships is not APIHelper.SKIP:
            self.client_memberships = client_memberships
        if client_arrivals is not APIHelper.SKIP:
            self.client_arrivals = client_arrivals

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        client =\
            ClientWithSuspensionInfo.from_dictionary(
                dictionary.get("Client"))\
                if "Client" in dictionary.keys()\
                else APIHelper.SKIP
        client_services = None
        if dictionary.get("ClientServices") is not None:
            client_services = [
                ClientService.from_dictionary(x)
                    for x in dictionary.get("ClientServices")
            ]
        else:
            client_services = APIHelper.SKIP
        client_contracts = None
        if dictionary.get("ClientContracts") is not None:
            client_contracts = [
                ClientContract.from_dictionary(x)
                    for x in dictionary.get("ClientContracts")
            ]
        else:
            client_contracts = APIHelper.SKIP
        client_memberships = None
        if dictionary.get("ClientMemberships") is not None:
            client_memberships = [
                ClientMembership.from_dictionary(x)
                    for x in dictionary.get("ClientMemberships")
            ]
        else:
            client_memberships = APIHelper.SKIP
        client_arrivals = None
        if dictionary.get("ClientArrivals") is not None:
            client_arrivals = [
                ClientArrival.from_dictionary(x)
                    for x in dictionary.get("ClientArrivals")
            ]
        else:
            client_arrivals = APIHelper.SKIP

        # Return an object of this model
        return cls(client,
                   client_services,
                   client_contracts,
                   client_memberships,
                   client_arrivals)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _client=(
            self.client
            if hasattr(self, "client")
            else None
        )
        _client_services=(
            self.client_services
            if hasattr(self, "client_services")
            else None
        )
        _client_contracts=(
            self.client_contracts
            if hasattr(self, "client_contracts")
            else None
        )
        _client_memberships=(
            self.client_memberships
            if hasattr(self, "client_memberships")
            else None
        )
        _client_arrivals=(
            self.client_arrivals
            if hasattr(self, "client_arrivals")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"client={_client!r}, "
            f"client_services={_client_services!r}, "
            f"client_contracts={_client_contracts!r}, "
            f"client_memberships={_client_memberships!r}, "
            f"client_arrivals={_client_arrivals!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _client=(
            self.client
            if hasattr(self, "client")
            else None
        )
        _client_services=(
            self.client_services
            if hasattr(self, "client_services")
            else None
        )
        _client_contracts=(
            self.client_contracts
            if hasattr(self, "client_contracts")
            else None
        )
        _client_memberships=(
            self.client_memberships
            if hasattr(self, "client_memberships")
            else None
        )
        _client_arrivals=(
            self.client_arrivals
            if hasattr(self, "client_arrivals")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"client={_client!s}, "
            f"client_services={_client_services!s}, "
            f"client_contracts={_client_contracts!s}, "
            f"client_memberships={_client_memberships!s}, "
            f"client_arrivals={_client_arrivals!s}, "
            f")"
        )



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.api_helper import APIHelper


class CrossRegionalClientAssociation(object):
    """Implementation of the 'CrossRegionalClientAssociation' model.

    A client cross region association

    Attributes:
        site_id (int): The ID of the site to which the client belongs.
        client_id (str): The client’s RSSID.
        unique_id (int): The client’s unique ID.
        site_is_active (bool): Indicates if site is active

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "site_id": "SiteId",
        "client_id": "ClientId",
        "unique_id": "UniqueId",
        "site_is_active": "SiteIsActive",
    }

    _optionals = [
        "site_id",
        "client_id",
        "unique_id",
        "site_is_active",
    ]

    def __init__(
        self,
        site_id=APIHelper.SKIP,
        client_id=APIHelper.SKIP,
        unique_id=APIHelper.SKIP,
        site_is_active=APIHelper.SKIP):
        """Initialize a CrossRegionalClientAssociation instance."""
        # Initialize members of the class
        if site_id is not APIHelper.SKIP:
            self.site_id = site_id
        if client_id is not APIHelper.SKIP:
            self.client_id = client_id
        if unique_id is not APIHelper.SKIP:
            self.unique_id = unique_id
        if site_is_active is not APIHelper.SKIP:
            self.site_is_active = site_is_active

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        site_id =\
            dictionary.get("SiteId")\
            if dictionary.get("SiteId")\
                else APIHelper.SKIP
        client_id =\
            dictionary.get("ClientId")\
            if dictionary.get("ClientId")\
                else APIHelper.SKIP
        unique_id =\
            dictionary.get("UniqueId")\
            if dictionary.get("UniqueId")\
                else APIHelper.SKIP
        site_is_active =\
            dictionary.get("SiteIsActive")\
            if "SiteIsActive" in dictionary.keys()\
                else APIHelper.SKIP

        # Return an object of this model
        return cls(site_id,
                   client_id,
                   unique_id,
                   site_is_active)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _site_id=(
            self.site_id
            if hasattr(self, "site_id")
            else None
        )
        _client_id=(
            self.client_id
            if hasattr(self, "client_id")
            else None
        )
        _unique_id=(
            self.unique_id
            if hasattr(self, "unique_id")
            else None
        )
        _site_is_active=(
            self.site_is_active
            if hasattr(self, "site_is_active")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"site_id={_site_id!r}, "
            f"client_id={_client_id!r}, "
            f"unique_id={_unique_id!r}, "
            f"site_is_active={_site_is_active!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _site_id=(
            self.site_id
            if hasattr(self, "site_id")
            else None
        )
        _client_id=(
            self.client_id
            if hasattr(self, "client_id")
            else None
        )
        _unique_id=(
            self.unique_id
            if hasattr(self, "unique_id")
            else None
        )
        _site_is_active=(
            self.site_is_active
            if hasattr(self, "site_is_active")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"site_id={_site_id!s}, "
            f"client_id={_client_id!s}, "
            f"unique_id={_unique_id!s}, "
            f"site_is_active={_site_is_active!s}, "
            f")"
        )



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: D410, E501, E101, D206
from apimatic_core.authentication.multiple.single_auth import (
    Single,
)
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter

from mindbodypublicapi.api_helper import APIHelper
from mindbodypublicapi.configuration import (
    Server,
)
from mindbodypublicapi.controllers.base_controller import (
    BaseController,
)
from mindbodypublicapi.http.http_method_enum import (
    HttpMethodEnum,
)
from mindbodypublicapi.models.add_client_to_class_response import (
    AddClientToClassResponse,
)
from mindbodypublicapi.models.cancel_single_class_response import (
    CancelSingleClassResponse,
)
from mindbodypublicapi.models.get_class_descriptions_response import (
    GetClassDescriptionsResponse,
)
from mindbodypublicapi.models.get_class_schedules_response import (
    GetClassSchedulesResponse,
)
from mindbodypublicapi.models.get_class_visits_response import (
    GetClassVisitsResponse,
)
from mindbodypublicapi.models.get_classes_response import (
    GetClassesResponse,
)
from mindbodypublicapi.models.get_courses_reponse import (
    GetCoursesReponse,
)
from mindbodypublicapi.models.get_semesters_response import (
    GetSemestersResponse,
)
from mindbodypublicapi.models.get_waitlist_entries_response import (
    GetWaitlistEntriesResponse,
)
from mindbodypublicapi.models.remove_client_from_class_response import (
    RemoveClientFromClassResponse,
)
from mindbodypublicapi.models.remove_clients_from_classes_response import (
    RemoveClientsFromClassesResponse,
)
from mindbodypublicapi.models.substitute_class_teacher_response import (
    SubstituteClassTeacherResponse,
)
from mindbodypublicapi.models.written_class_schedules_info import (
    WrittenClassSchedulesInfo,
)


class ClassController(BaseController):
    """A Controller to access Endpoints in the mindbodypublicapi API."""

    def __init__(self, config):
        """Initialize ClassController object."""
        super(ClassController, self).__init__(config)

    def get_class_descriptions(self,
                               version,
                               site_id,
                               authorization=None,
                               request_class_description_id=None,
                               request_end_class_date_time=None,
                               request_include_inactive=None,
                               request_limit=None,
                               request_location_id=None,
                               request_offset=None,
                               request_program_ids=None,
                               request_staff_id=None,
                               request_start_class_date_time=None):
        """Perform a GET request to /public/v{version}/class/classdescriptions.

        To find class descriptions associated with **scheduled classes**, pass
        `StaffId`, `StartClassDateTime`, `EndClassDateTime`, or `LocationId` in the
        request.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_class_description_id (int, optional): The ID of the requested
                client.
            request_end_class_date_time (datetime, optional): Filters the results to
                class descriptions for scheduled classes that happen before the given
                date and time.
            request_include_inactive (bool, optional): Includes inactive class
                descriptions, defaulting to true. When set to false, it filters out
                inactive class descriptions.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_id (int, optional): Filters results to classes
                descriptions for schedule classes as the given location.
            request_offset (int, optional): Page offset, defaults to 0.
            request_program_ids (List[int], optional): A list of requested program
                IDs.
            request_staff_id (int, optional): Filters results to class descriptions
                for scheduled classes taught by the given staff member.
            request_start_class_date_time (datetime, optional): Filters the results
                to class descriptions for scheduled classes that happen on or after
                the given date and time.

        Returns:
            GetClassDescriptionsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/classdescriptions")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.classDescriptionId")
                .value(request_class_description_id))
            .query_param(Parameter()
                .key("request.endClassDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_end_class_date_time)))
            .query_param(Parameter()
                .key("request.includeInactive")
                .value(request_include_inactive))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationId")
                .value(request_location_id))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.programIds")
                .value(request_program_ids))
            .query_param(Parameter()
                .key("request.staffId")
                .value(request_staff_id))
            .query_param(Parameter()
                .key("request.startClassDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_start_class_date_time)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetClassDescriptionsResponse.from_dictionary),
        ).execute()

    def get_classes(self,
                    version,
                    site_id,
                    authorization=None,
                    request_class_description_ids=None,
                    request_class_ids=None,
                    request_class_schedule_ids=None,
                    request_client_id=None,
                    request_end_date_time=None,
                    request_hide_canceled_classes=None,
                    request_last_modified_date=None,
                    request_limit=None,
                    request_location_ids=None,
                    request_offset=None,
                    request_program_ids=None,
                    request_scheduling_window=None,
                    request_semester_ids=None,
                    request_session_type_ids=None,
                    request_staff_ids=None,
                    request_start_date_time=None,
                    request_unique_client_id=None):
        """Perform a GET request to /public/v{version}/class/classes.

        Get scheduled classes.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_class_description_ids (List[int], optional): The requested class
                description IDs.
            request_class_ids (List[int], optional): The requested class IDs.
            request_class_schedule_ids (List[int], optional): The requested
                classSchedule Ids.
            request_client_id (str, optional): The client ID of the client who is
                viewing this class list. Based on identity, the client may be able to
                see additional information, such as membership specials.
            request_end_date_time (datetime, optional): The requested end date for
                filtering.  NOTE: ClassDate does not take Class Time into
                consideration.  <br />Default: **today’s date**
            request_hide_canceled_classes (bool, optional): When `true`, canceled
                classes are removed from the response.<br />  When `false`, canceled
                classes are included in the response.<br />  Default: **false**
            request_last_modified_date (datetime, optional): When included in the
                request, only records modified on or after the `LastModifiedDate`
                specified are included in the response.
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_ids (List[int], optional): A list of location IDs on
                which to base the search.
            request_offset (int, optional): Page offset, defaults to 0.
            request_program_ids (List[int], optional): A list of program IDs on which
                to base the search.
            request_scheduling_window (bool, optional): When `true`, classes outside
                scheduling window are removed from the response.<br />  When `false`,
                classes are included in the response, regardless of the scheduling
                window.<br />  Default: **false**
            request_semester_ids (List[int], optional): A list of semester IDs on
                which to base the search.
            request_session_type_ids (List[int], optional): A list of session type
                IDs on which to base the search.
            request_staff_ids (List[int], optional): The requested IDs of the
                teaching staff members.
            request_start_date_time (datetime, optional): The requested start date
                for filtering. This also determines what you will see for the
                ‘BookingWindow’ StartDateTime in the response. For example, if you
                pass a StartDateTime that is on OR before the BookingWindow ‘Open’
                days of the class, you will retrieve the actual ‘StartDateTime’ for
                the Booking Window. If you pass a StartDateTime that is after the
                BookingWindow ‘date’, then you will receive results based on that
                start date.  NOTE: ClassDate does not take Class Time into
                consideration.
            request_unique_client_id (int, optional): The unique ID of the client who
                is viewing this class list. Based on identity, the client may be able
                to see additional information, such as membership specials.  Note:
                you need to provide the 'UniqueClientId' OR the 'ClientId'. If both
                are provided, the 'UniqueClientId' takes precedence.

        Returns:
            GetClassesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/classes")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.classDescriptionIds")
                .value(request_class_description_ids))
            .query_param(Parameter()
                .key("request.classIds")
                .value(request_class_ids))
            .query_param(Parameter()
                .key("request.classScheduleIds")
                .value(request_class_schedule_ids))
            .query_param(Parameter()
                .key("request.clientId")
                .value(request_client_id))
            .query_param(Parameter()
                .key("request.endDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_end_date_time)))
            .query_param(Parameter()
                .key("request.hideCanceledClasses")
                .value(request_hide_canceled_classes))
            .query_param(Parameter()
                .key("request.lastModifiedDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_last_modified_date)))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationIds")
                .value(request_location_ids))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.programIds")
                .value(request_program_ids))
            .query_param(Parameter()
                .key("request.schedulingWindow")
                .value(request_scheduling_window))
            .query_param(Parameter()
                .key("request.semesterIds")
                .value(request_semester_ids))
            .query_param(Parameter()
                .key("request.sessionTypeIds")
                .value(request_session_type_ids))
            .query_param(Parameter()
                .key("request.staffIds")
                .value(request_staff_ids))
            .query_param(Parameter()
                .key("request.startDateTime")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_start_date_time)))
            .query_param(Parameter()
                .key("request.uniqueClientId")
                .value(request_unique_client_id))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetClassesResponse.from_dictionary),
        ).execute()

    def get_class_schedules(self,
                            version,
                            site_id,
                            authorization=None,
                            request_class_schedule_ids=None,
                            request_end_date=None,
                            request_limit=None,
                            request_location_ids=None,
                            request_offset=None,
                            request_program_ids=None,
                            request_session_type_ids=None,
                            request_staff_ids=None,
                            request_start_date=None):
        """Perform a GET request to /public/v{version}/class/classschedules.

        Get class schedules.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_class_schedule_ids (List[int], optional): The class schedule IDs.
                <br />Default: **all**
            request_end_date (datetime, optional): The end date of the range. Return
                any active enrollments that occur on or before this day.  <br
                />Default: **StartDate**
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_location_ids (List[int], optional): The location IDs.   <br
                />Default: **all**
            request_offset (int, optional): Page offset, defaults to 0.
            request_program_ids (List[int], optional): The program IDs.   <br
                />Default: **all**
            request_session_type_ids (List[int], optional): The session type IDs.
                <br />Default: **all**
            request_staff_ids (List[int], optional): The staff IDs.   <br />Default:
                **all**
            request_start_date (datetime, optional): The start date of the range.
                Return any active enrollments that occur on or after this day.  <br
                />Default: **today’s date**

        Returns:
            GetClassSchedulesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/classschedules")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.classScheduleIds")
                .value(request_class_schedule_ids))
            .query_param(Parameter()
                .key("request.endDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_end_date)))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.locationIds")
                .value(request_location_ids))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.programIds")
                .value(request_program_ids))
            .query_param(Parameter()
                .key("request.sessionTypeIds")
                .value(request_session_type_ids))
            .query_param(Parameter()
                .key("request.staffIds")
                .value(request_staff_ids))
            .query_param(Parameter()
                .key("request.startDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_start_date)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetClassSchedulesResponse.from_dictionary),
        ).execute()

    def get_class_visits(self,
                         version,
                         request_class_id,
                         site_id,
                         authorization=None,
                         request_last_modified_date=None,
                         request_use_site_settings_staff_name=None):
        """Perform a GET request to /public/v{version}/class/classvisits.

        Returns a list of visits that contain information for a specified class. On
        success, this request returns the class object in the response with a list of
        visits.

        Args:
            version (str): version of the api.
            request_class_id (int): The class ID.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_last_modified_date (datetime, optional): When included in the
                request, only records modified on or after the `LastModifiedDate`
                specified are included in the response.
            request_use_site_settings_staff_name (bool, optional): When `true`, the
                staff DisplayName will be populated based on site-level settings.
                When `false` or omitted, the staff DisplayName will contain only the
                FirstName.

        Returns:
            GetClassVisitsResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/classvisits")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .query_param(Parameter()
                .key("request.classID")
                .value(request_class_id))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.lastModifiedDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_last_modified_date)))
            .query_param(Parameter()
                .key("request.useSiteSettingsStaffName")
                .value(request_use_site_settings_staff_name))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetClassVisitsResponse.from_dictionary),
        ).execute()

    def get_courses(self,
                    version,
                    site_id,
                    authorization=None,
                    get_courses_request_course_i_ds=None,
                    get_courses_request_end_date=None,
                    get_courses_request_limit=None,
                    get_courses_request_location_i_ds=None,
                    get_courses_request_offset=None,
                    get_courses_request_program_i_ds=None,
                    get_courses_request_semester_i_ds=None,
                    get_courses_request_staff_i_ds=None,
                    get_courses_request_start_date=None):
        """Perform a GET request to /public/v{version}/class/courses.

        This endpoint will provide all the data related to courses depending on the
        access level.<br />
        Note: The Authorization is an optional header.If Authorization header is not
        passed, the response will be masked else full response will be provided.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            get_courses_request_course_i_ds (List[int], optional): Return only
                courses that are available for the specified CourseIds.
            get_courses_request_end_date (datetime, optional): The end date range.
                Any active courses that are on or before this day.  <br />(optional)
                Defaults to StartDate.
            get_courses_request_limit (int, optional): Number of results to include,
                defaults to 100
            get_courses_request_location_i_ds (List[int], optional): Return only
                courses that are available for the specified LocationIds.
            get_courses_request_offset (int, optional): Page offset, defaults to 0.
            get_courses_request_program_i_ds (List[int], optional): Return only
                courses that are available for the specified ProgramIds.
            get_courses_request_semester_i_ds (List[int], optional): Return only
                courses that are available for the specified SemesterIds.
            get_courses_request_staff_i_ds (List[int], optional): Return only courses
                that are available for the specified StaffIds.
            get_courses_request_start_date (datetime, optional): The start date
                range. Any active courses that are on or after this day.  <br
                />(optional) Defaults to today.

        Returns:
            GetCoursesReponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/courses")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("getCoursesRequest.courseIDs")
                .value(get_courses_request_course_i_ds))
            .query_param(Parameter()
                .key("getCoursesRequest.endDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    get_courses_request_end_date)))
            .query_param(Parameter()
                .key("getCoursesRequest.limit")
                .value(get_courses_request_limit))
            .query_param(Parameter()
                .key("getCoursesRequest.locationIDs")
                .value(get_courses_request_location_i_ds))
            .query_param(Parameter()
                .key("getCoursesRequest.offset")
                .value(get_courses_request_offset))
            .query_param(Parameter()
                .key("getCoursesRequest.programIDs")
                .value(get_courses_request_program_i_ds))
            .query_param(Parameter()
                .key("getCoursesRequest.semesterIDs")
                .value(get_courses_request_semester_i_ds))
            .query_param(Parameter()
                .key("getCoursesRequest.staffIDs")
                .value(get_courses_request_staff_i_ds))
            .query_param(Parameter()
                .key("getCoursesRequest.startDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    get_courses_request_start_date)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetCoursesReponse.from_dictionary),
        ).execute()

    def get_semesters(self,
                      version,
                      site_id,
                      authorization=None,
                      request_active=None,
                      request_end_date=None,
                      request_limit=None,
                      request_offset=None,
                      request_semester_i_ds=None,
                      request_start_date=None):
        """Perform a GET request to /public/v{version}/class/semesters.

        This endpoint retrieves the business class semesters.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_active (bool, optional): When true, the response only contains
                semesters which are activated. When false, only deactivated semesters
                are returned.  Default: **All semesters**
            request_end_date (datetime, optional): The end date for the range. All
                semesters that are on or before this day.  Default: **StartDate**
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_offset (int, optional): Page offset, defaults to 0.
            request_semester_i_ds (List[int], optional): The requested semester IDs.
            request_start_date (datetime, optional): The start date for the range.
                All semesters that are on or after this day.  Default: **today’s
                date**

        Returns:
            GetSemestersResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/semesters")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.active")
                .value(request_active))
            .query_param(Parameter()
                .key("request.endDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_end_date)))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.semesterIDs")
                .value(request_semester_i_ds))
            .query_param(Parameter()
                .key("request.startDate")
                .value(APIHelper.when_defined(APIHelper.RFC3339DateTime,
                    request_start_date)))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetSemestersResponse.from_dictionary),
        ).execute()

    def get_waitlist_entries(self,
                             version,
                             site_id,
                             authorization=None,
                             request_class_ids=None,
                             request_class_schedule_ids=None,
                             request_client_ids=None,
                             request_hide_past_entries=None,
                             request_limit=None,
                             request_offset=None,
                             request_waitlist_entry_ids=None):
        """Perform a GET request to /public/v{version}/class/waitlistentries.

        Returns a list of waiting list entries for a specified class schedule or
        class. The request requires staff credentials and either a class schedule ID
        or class ID.

        Args:
            version (str): version of the api.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.
            request_class_ids (List[int], optional): The requested class IDs. If a
                class ID is present, the request automatically disregards any class
                schedule IDs in the request. <br />  Either `ClassScheduleIds`,
                `ClientIds`, `WaitlistEntryIds`, or `ClassIds` is required; the
                others become optional.<br />  Default: **all ClassIds**
            request_class_schedule_ids (List[int], optional): The requested class
                schedule IDs. If a class ID is present, the request automatically
                disregards any class schedule IDs in the request.<br />  Either
                `ClassScheduleIds`, `ClientIds`, `WaitlistEntryIds`, or `ClassIds` is
                required; the others become optional.<br />  Default: **all
                ClassScheduleIds**
            request_client_ids (List[str], optional): The requested client IDs.<br />
                Either `ClassScheduleIds`, `ClientIds`, `WaitlistEntryIds`, or
                `ClassIds` is required; the others become optional.<br />  Default:
                **all ClientIds**
            request_hide_past_entries (bool, optional): When `true`, indicates that
                past waiting list entries are hidden from clients.<br />  When
                `false`, indicates that past entries are not hidden from clients.<br
                />  Default: **false**
            request_limit (int, optional): Number of results to include, defaults to
                100
            request_offset (int, optional): Page offset, defaults to 0.
            request_waitlist_entry_ids (List[int], optional): The requested waiting
                list entry IDs.<br />  Either `ClassScheduleIds`, `ClientIds`,
                `WaitlistEntryIds`, or `ClassIds` is required; the others become
                optional.<br />  Default: **all WaitlistEntryIds**

        Returns:
            GetWaitlistEntriesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/waitlistentries")
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .query_param(Parameter()
                .key("request.classIds")
                .value(request_class_ids))
            .query_param(Parameter()
                .key("request.classScheduleIds")
                .value(request_class_schedule_ids))
            .query_param(Parameter()
                .key("request.clientIds")
                .value(request_client_ids))
            .query_param(Parameter()
                .key("request.hidePastEntries")
                .value(request_hide_past_entries))
            .query_param(Parameter()
                .key("request.limit")
                .value(request_limit))
            .query_param(Parameter()
                .key("request.offset")
                .value(request_offset))
            .query_param(Parameter()
                .key("request.waitlistEntryIds")
                .value(request_waitlist_entry_ids))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(GetWaitlistEntriesResponse.from_dictionary),
        ).execute()

    def add_class_schedule(self,
                           version,
                           request,
                           site_id,
                           authorization=None):
        """Perform a POST request to /public/v{version}/class/addclassschedule.

        This endpoint adds a class schedule. For a single day schedule, the EndDate
        parameter can be omitted.

        Args:
            version (str): version of the api.
            request (AddClassEnrollmentScheduleRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            WrittenClassSchedulesInfo: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/addclassschedule")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(WrittenClassSchedulesInfo.from_dictionary),
        ).execute()

    def add_client_to_class(self,
                            version,
                            request,
                            site_id,
                            authorization=None):
        """Perform a POST request to /public/v{version}/class/addclienttoclass.

        This endpoint adds a client to a class or to a class waiting list. To prevent
        overbooking a class or booking outside the schedule windows set forth by the
        business, it is necessary to first check the capacity level of the class
        (‘MaxCapacity’ and 'TotalBooked’) and the 'IsAvailable’ parameter by running
        the GetClasses REQUEST. It is helpful to use this endpoint in the following
        situations:
        * Use after calling `GET Clients` and `GET Classes` so that you are sure
        which client to book in which class.
        * If adding a client to a class from a waiting list, use this call after you
        call `GET WaitlistEntries` and determine the ID of the waiting list from
        which you are moving the client.
        * If adding a client to a class and using a pricing option that the client
        has already purchased, use this call after you call `GET ClientServices` to
        determine the ID of the pricing option that the client wants to use.
        If you add a client to a class and the client purchases a new pricing option,
        use `GET Services`, `GET Classes`, and then `POST CheckoutShoppingCart` in
        place of this call.
        This endpoint also supports cross-regional class bookings. If you want to
        perform a cross-regional class booking, set `CrossRegionalBooking` to `true`.
        This endpoint does not support adding a user to a waiting list using a
        cross-regional client pricing option(service). Cross-regional booking
        workflows do not support client service scheduling restrictions.
        When performing a cross-regional class booking, this endpoint loops through
        the first ten sites that the client is associated with, looks for client
        pricing options at each of those sites, and then uses the oldest client
        pricing option found.It is important to note that this endpoint only loops
        through a maximum of ten associated client sites. If a `ClientID` is
        associated with more than ten sites in an organization, this endpoint only
        loops through the first ten.If you know that a client has a client service at
        another site, you can specify that site using the
        `CrossRegionalBookingClientServiceSiteId` query parameter.
        If you perform a cross-regional booking, two additional fields are included
        in the `SessionType` object of the response:
        * `SiteID`, which specifies where the client service is coming from
        * `CrossRegionalBookingPerformed`, a Boolean field that is set to `true`
        As a prerequisite to using this endpoint, your `SourceName` must have been
        granted access to the organization to which the site belongs.

        Note: Request deduplication is enabled for this endpoint. More information
        can be found in the [Request Deduplication
        Page](https://developers.mindbodyonline.com/ui/documentation/public-api#/net-s
        tandard-library/mindbody-public-api-v6-0/request-deduplication).
        Use the `X-RequestDeduplication-Skip` header to bypass deduplication if
        necessary. This may be useful in scenarios where you want to ensure a new
        appointment is created regardless of previous identical requests.

        Args:
            version (str): version of the api.
            request (AddClientToClassRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            AddClientToClassResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/addclienttoclass")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(AddClientToClassResponse.from_dictionary),
        ).execute()

    def cancel_single_class(self,
                            version,
                            request,
                            site_id,
                            authorization=None):
        """Perform a POST request to
        /public/v{version}/class/cancelsingleclass.

        This endpoint will cancel a single class from studio.

        Args:
            version (str): version of the api.
            request (CancelSingleClassRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            CancelSingleClassResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/cancelsingleclass")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CancelSingleClassResponse.from_dictionary),
        ).execute()

    def remove_client_from_class(self,
                                 version,
                                 request,
                                 site_id,
                                 authorization=None):
        """Perform a POST request to
        /public/v{version}/class/removeclientfromclass.

        Remove a client from a class.

        Args:
            version (str): version of the api.
            request (RemoveClientFromClassRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            RemoveClientFromClassResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/removeclientfromclass")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(RemoveClientFromClassResponse.from_dictionary),
        ).execute()

    def remove_clients_from_classes(self,
                                    version,
                                    request,
                                    site_id,
                                    authorization=None):
        """Perform a POST request to
        /public/v{version}/class/removeclientsfromclasses.

        This endpoint can be utilized for removing multiple clients from multiple
        classes in one request.

        Args:
            version (str): version of the api.
            request (RemoveClientsFromClassesRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            RemoveClientsFromClassesResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/removeclientsfromclasses")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(RemoveClientsFromClassesResponse.from_dictionary),
        ).execute()

    def remove_from_waitlist(self,
                             version,
                             request_waitlist_entry_ids,
                             site_id,
                             authorization=None):
        """Perform a POST request to
        /public/v{version}/class/removefromwaitlist.

        This endpoint does not return a response. If a call to this endpoint results
        in a 200 OK HTTP status code, then the call was successful.

        Args:
            version (str): version of the api.
            request_waitlist_entry_ids (List[int]): A list of `WaitlistEntryIds` to
                remove from the waiting list.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            Any: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/removefromwaitlist")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .query_param(Parameter()
                .key("request.waitlistEntryIds")
                .value(request_waitlist_entry_ids))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize),
        ).execute()

    def substitute_class_teacher(self,
                                 version,
                                 request,
                                 site_id,
                                 authorization=None):
        """Perform a POST request to
        /public/v{version}/class/substituteclassteacher.

        Substitute a class teacher.

        Args:
            version (str): version of the api.
            request (SubstituteClassTeacherRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            SubstituteClassTeacherResponse: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/substituteclassteacher")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SubstituteClassTeacherResponse.from_dictionary),
        ).execute()

    def update_class_schedule(self,
                              version,
                              request,
                              site_id,
                              authorization=None):
        """Perform a POST request to
        /public/v{version}/class/updateclassschedule.

        This endpoint updates a class schedule.

        Args:
            version (str): version of the api.
            request (UpdateClassEnrollmentScheduleRequest): The request body
                parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            WrittenClassSchedulesInfo: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/updateclassschedule")
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .header_param(Parameter()
                .key("accept")
                .value("application/json"))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(WrittenClassSchedulesInfo.from_dictionary),
        ).execute()

    def update_class_schedule_notes(self,
                                    version,
                                    class_schedule_id,
                                    request,
                                    site_id,
                                    authorization=None):
        """Perform a PATCH request to
        /public/v{version}/class/updateclassschedulenotes/{classScheduleId}.

        This endpoint updates the notes of class instances based on the schedule's
        schedule ID.
        Note: Every coming class instance for the given ScheduleID will have the
        notes updated the same way.

        Args:
            version (str): version of the api.
            class_schedule_id (int): The request template parameter.
            request (UpdateClassScheduleNotesRequest): The request body parameter.
            site_id (str): ID of the site from which to pull data.
            authorization (str, optional): A staff user authorization token.

        Returns:
            void: Response from the API. No Content

        Raises:
            APIException: When an error occurs while fetching the data from the
                remote API. This exception includes the HTTP Response code, an error
                message, and the HTTP body that was received in the request.

        """
        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path("/public/v{version}/class/updateclassschedulenotes/{classScheduleId}")
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                .key("version")
                .value(version)
                .should_encode(True))
            .template_param(Parameter()
                .key("classScheduleId")
                .value(class_schedule_id)
                .should_encode(True))
            .body_param(Parameter()
                .value(request))
            .header_param(Parameter()
                .key("siteId")
                .value(site_id))
            .header_param(Parameter()
                .key("Content-Type")
                .value("application/json"))
            .header_param(Parameter()
                .key("authorization")
                .value(authorization))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single("API-Key")),
        ).execute()



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.api_helper import APIHelper


class Tip(object):
    """Implementation of the 'Tip' model.

    Attributes:
        staff_id (int): The ID of the staff member the tip is for.
        sale_id (int): The sale’s ID associated with the tip.
        sale_date_time (datetime): The date and time when the tip was given.
        earnings (float): The amount tipped to the staff member.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "staff_id": "StaffId",
        "sale_id": "SaleId",
        "sale_date_time": "SaleDateTime",
        "earnings": "Earnings",
    }

    _optionals = [
        "staff_id",
        "sale_id",
        "sale_date_time",
        "earnings",
    ]

    def __init__(
        self,
        staff_id=APIHelper.SKIP,
        sale_id=APIHelper.SKIP,
        sale_date_time=APIHelper.SKIP,
        earnings=APIHelper.SKIP):
        """Initialize a Tip instance."""
        # Initialize members of the class
        if staff_id is not APIHelper.SKIP:
            self.staff_id = staff_id
        if sale_id is not APIHelper.SKIP:
            self.sale_id = sale_id
        if sale_date_time is not APIHelper.SKIP:
            self.sale_date_time =\
                 APIHelper.apply_datetime_converter(
                sale_date_time, APIHelper.RFC3339DateTime)\
                 if sale_date_time else None
        if earnings is not APIHelper.SKIP:
            self.earnings = earnings

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        staff_id =\
            dictionary.get("StaffId")\
            if dictionary.get("StaffId")\
                else APIHelper.SKIP
        sale_id =\
            dictionary.get("SaleId")\
            if dictionary.get("SaleId")\
                else APIHelper.SKIP
        sale_date_time = APIHelper.RFC3339DateTime.from_value(
            dictionary.get("SaleDateTime")).datetime\
            if dictionary.get("SaleDateTime") else APIHelper.SKIP
        earnings =\
            dictionary.get("Earnings")\
            if dictionary.get("Earnings")\
                else APIHelper.SKIP

        # Return an object of this model
        return cls(staff_id,
                   sale_id,
                   sale_date_time,
                   earnings)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _staff_id=(
            self.staff_id
            if hasattr(self, "staff_id")
            else None
        )
        _sale_id=(
            self.sale_id
            if hasattr(self, "sale_id")
            else None
        )
        _sale_date_time=(
            self.sale_date_time
            if hasattr(self, "sale_date_time")
            else None
        )
        _earnings=(
            self.earnings
            if hasattr(self, "earnings")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"staff_id={_staff_id!r}, "
            f"sale_id={_sale_id!r}, "
            f"sale_date_time={_sale_date_time!r}, "
            f"earnings={_earnings!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _staff_id=(
            self.staff_id
            if hasattr(self, "staff_id")
            else None
        )
        _sale_id=(
            self.sale_id
            if hasattr(self, "sale_id")
            else None
        )
        _sale_date_time=(
            self.sale_date_time
            if hasattr(self, "sale_date_time")
            else None
        )
        _earnings=(
            self.earnings
            if hasattr(self, "earnings")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"staff_id={_staff_id!s}, "
            f"sale_id={_sale_id!s}, "
            f"sale_date_time={_sale_date_time!s}, "
            f"earnings={_earnings!s}, "
            f")"
        )



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.api_helper import APIHelper


class UpdatePricingOptionRequest(object):
    """Implementation of the 'UpdatePricingOptionRequest' model.

    To Update pricing option data

    Attributes:
        product_id (float): The id of the pricing option (legacy Id in mongo db)
        name (str): Pricing option name.
        price (float): Price of this Pricing option.
        online_price (float): Online price of this Pricing option.
        count (int): Number of sessions for this pricing option
        sell_online (bool): Whether this pricing option sell online or not
        revenue_category (str): Revenue Category of this pricing option
        discontinued (bool): Whether this pricing option active or inactive
        membership_id (int): The ID of the membership required to purchase the
            pricing option  Default is -1 and will not update Membership if it is not
            passed in body
        is_third_party_discount_pricing (bool): Is this pricing option sold at
            discounted rates by third-parties  Once set to true it cannot be changed
        priority (str): The priority of the pricing option  Priority can be set to
            Low(-1), Medium (0), High(1)
        sell_at_location_ids (List[int]): The location IDs where this pricing option
            is sold   (default is null and will not update SellAtLocation if it is
            not passed in body)
        use_at_location_ids (List[int]): The location IDs where this pricing option
            is used  (default is null and will not update UseAtLocation if it is not
            passed in body)
        expiration_unit (str): The Expiration unit, either Days or Months
        expiration_length (int): The number of days or months that the pricing option
            is active for.
        expiration_type (str): The date the pricing option begins its activation,
            either the SaleDate or DateOfClientFirstVisit
        restricted_membership_ids (List[int]): This pricing option can be used under
            these memberships only  If null/empty then not restricted to any
            membership

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "product_id": "ProductId",
        "name": "Name",
        "price": "Price",
        "online_price": "OnlinePrice",
        "count": "Count",
        "sell_online": "SellOnline",
        "revenue_category": "RevenueCategory",
        "discontinued": "Discontinued",
        "membership_id": "MembershipId",
        "is_third_party_discount_pricing": "IsThirdPartyDiscountPricing",
        "priority": "Priority",
        "sell_at_location_ids": "SellAtLocationIds",
        "use_at_location_ids": "UseAtLocationIds",
        "expiration_unit": "ExpirationUnit",
        "expiration_length": "ExpirationLength",
        "expiration_type": "ExpirationType",
        "restricted_membership_ids": "RestrictedMembershipIds",
    }

    _optionals = [
        "name",
        "price",
        "online_price",
        "count",
        "sell_online",
        "revenue_category",
        "discontinued",
        "membership_id",
        "is_third_party_discount_pricing",
        "priority",
        "sell_at_location_ids",
        "use_at_location_ids",
        "expiration_unit",
        "expiration_length",
        "expiration_type",
        "restricted_membership_ids",
    ]

    def __init__(
        self,
        product_id=None,
        name=APIHelper.SKIP,
        price=APIHelper.SKIP,
        online_price=APIHelper.SKIP,
        count=APIHelper.SKIP,
        sell_online=APIHelper.SKIP,
        revenue_category=APIHelper.SKIP,
        discontinued=APIHelper.SKIP,
        membership_id=APIHelper.SKIP,
        is_third_party_discount_pricing=APIHelper.SKIP,
        priority=APIHelper.SKIP,
        sell_at_location_ids=APIHelper.SKIP,
        use_at_location_ids=APIHelper.SKIP,
        expiration_unit=APIHelper.SKIP,
        expiration_length=APIHelper.SKIP,
        expiration_type=APIHelper.SKIP,
        restricted_membership_ids=APIHelper.SKIP):
        """Initialize a UpdatePricingOptionRequest instance."""
        # Initialize members of the class
        self.product_id = product_id
        if name is not APIHelper.SKIP:
            self.name = name
        if price is not APIHelper.SKIP:
            self.price = price
        if online_price is not APIHelper.SKIP:
            self.online_price = online_price
        if count is not APIHelper.SKIP:
            self.count = count
        if sell_online is not APIHelper.SKIP:
            self.sell_online = sell_online
        if revenue_category is not APIHelper.SKIP:
            self.revenue_category = revenue_category
        if discontinued is not APIHelper.SKIP:
            self.discontinued = discontinued
        if membership_id is not APIHelper.SKIP:
            self.membership_id = membership_id
        if is_third_party_discount_pricing is not APIHelper.SKIP:
            self.is_third_party_discount_pricing = is_third_party_discount_pricing
        if priority is not APIHelper.SKIP:
            self.priority = priority
        if sell_at_location_ids is not APIHelper.SKIP:
            self.sell_at_location_ids = sell_at_location_ids
        if use_at_location_ids is not APIHelper.SKIP:
            self.use_at_location_ids = use_at_location_ids
        if expiration_unit is not APIHelper.SKIP:
            self.expiration_unit = expiration_unit
        if expiration_length is not APIHelper.SKIP:
            self.expiration_length = expiration_length
        if expiration_type is not APIHelper.SKIP:
            self.expiration_type = expiration_type
        if restricted_membership_ids is not APIHelper.SKIP:
            self.restricted_membership_ids = restricted_membership_ids

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        product_id =\
            dictionary.get("ProductId")\
            if dictionary.get("ProductId")\
                else None
        name =\
            dictionary.get("Name")\
            if dictionary.get("Name")\
                else APIHelper.SKIP
        price =\
            dictionary.get("Price")\
            if dictionary.get("Price")\
                else APIHelper.SKIP
        online_price =\
            dictionary.get("OnlinePrice")\
            if dictionary.get("OnlinePrice")\
                else APIHelper.SKIP
        count =\
            dictionary.get("Count")\
            if dictionary.get("Count")\
                else APIHelper.SKIP
        sell_online =\
            dictionary.get("SellOnline")\
            if "SellOnline" in dictionary.keys()\
                else APIHelper.SKIP
        revenue_category =\
            dictionary.get("RevenueCategory")\
            if dictionary.get("RevenueCategory")\
                else APIHelper.SKIP
        discontinued =\
            dictionary.get("Discontinued")\
            if "Discontinued" in dictionary.keys()\
                else APIHelper.SKIP
        membership_id =\
            dictionary.get("MembershipId")\
            if dictionary.get("MembershipId")\
                else APIHelper.SKIP
        is_third_party_discount_pricing =\
            dictionary.get("IsThirdPartyDiscountPricing")\
            if "IsThirdPartyDiscountPricing" in dictionary.keys()\
                else APIHelper.SKIP
        priority =\
            dictionary.get("Priority")\
            if dictionary.get("Priority")\
                else APIHelper.SKIP
        sell_at_location_ids =\
            dictionary.get("SellAtLocationIds")\
            if dictionary.get("SellAtLocationIds")\
                else APIHelper.SKIP
        use_at_location_ids =\
            dictionary.get("UseAtLocationIds")\
            if dictionary.get("UseAtLocationIds")\
                else APIHelper.SKIP
        expiration_unit =\
            dictionary.get("ExpirationUnit")\
            if dictionary.get("ExpirationUnit")\
                else APIHelper.SKIP
        expiration_length =\
            dictionary.get("ExpirationLength")\
            if dictionary.get("ExpirationLength")\
                else APIHelper.SKIP
        expiration_type =\
            dictionary.get("ExpirationType")\
            if dictionary.get("ExpirationType")\
                else APIHelper.SKIP
        restricted_membership_ids =\
            dictionary.get("RestrictedMembershipIds")\
            if dictionary.get("RestrictedMembershipIds")\
                else APIHelper.SKIP

        # Return an object of this model
        return cls(product_id,
                   name,
                   price,
                   online_price,
                   count,
                   sell_online,
                   revenue_category,
                   discontinued,
                   membership_id,
                   is_third_party_discount_pricing,
                   priority,
                   sell_at_location_ids,
                   use_at_location_ids,
                   expiration_unit,
                   expiration_length,
                   expiration_type,
                   restricted_membership_ids)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _product_id=self.product_id
        _name=(
            self.name
            if hasattr(self, "name")
            else None
        )
        _price=(
            self.price
            if hasattr(self, "price")
            else None
        )
        _online_price=(
            self.online_price
            if hasattr(self, "online_price")
            else None
        )
        _count=(
            self.count
            if hasattr(self, "count")
            else None
        )
        _sell_online=(
            self.sell_online
            if hasattr(self, "sell_online")
            else None
        )
        _revenue_category=(
            self.revenue_category
            if hasattr(self, "revenue_category")
            else None
        )
        _discontinued=(
            self.discontinued
            if hasattr(self, "discontinued")
            else None
        )
        _membership_id=(
            self.membership_id
            if hasattr(self, "membership_id")
            else None
        )
        _is_third_party_discount_pricing=(
            self.is_third_party_discount_pricing
            if hasattr(self, "is_third_party_discount_pricing")
            else None
        )
        _priority=(
            self.priority
            if hasattr(self, "priority")
            else None
        )
        _sell_at_location_ids=(
            self.sell_at_location_ids
            if hasattr(self, "sell_at_location_ids")
            else None
        )
        _use_at_location_ids=(
            self.use_at_location_ids
            if hasattr(self, "use_at_location_ids")
            else None
        )
        _expiration_unit=(
            self.expiration_unit
            if hasattr(self, "expiration_unit")
            else None
        )
        _expiration_length=(
            self.expiration_length
            if hasattr(self, "expiration_length")
            else None
        )
        _expiration_type=(
            self.expiration_type
            if hasattr(self, "expiration_type")
            else None
        )
        _restricted_membership_ids=(
            self.restricted_membership_ids
            if hasattr(self, "restricted_membership_ids")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"product_id={_product_id!r}, "
            f"name={_name!r}, "
            f"price={_price!r}, "
            f"online_price={_online_price!r}, "
            f"count={_count!r}, "
            f"sell_online={_sell_online!r}, "
            f"revenue_category={_revenue_category!r}, "
            f"discontinued={_discontinued!r}, "
            f"membership_id={_membership_id!r}, "
            f"is_third_party_discount_pricing={_is_third_party_discount_pricing!r}, "
            f"priority={_priority!r}, "
            f"sell_at_location_ids={_sell_at_location_ids!r}, "
            f"use_at_location_ids={_use_at_location_ids!r}, "
            f"expiration_unit={_expiration_unit!r}, "
            f"expiration_length={_expiration_length!r}, "
            f"expiration_type={_expiration_type!r}, "
            f"restricted_membership_ids={_restricted_membership_ids!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _product_id=self.product_id
        _name=(
            self.name
            if hasattr(self, "name")
            else None
        )
        _price=(
            self.price
            if hasattr(self, "price")
            else None
        )
        _online_price=(
            self.online_price
            if hasattr(self, "online_price")
            else None
        )
        _count=(
            self.count
            if hasattr(self, "count")
            else None
        )
        _sell_online=(
            self.sell_online
            if hasattr(self, "sell_online")
            else None
        )
        _revenue_category=(
            self.revenue_category
            if hasattr(self, "revenue_category")
            else None
        )
        _discontinued=(
            self.discontinued
            if hasattr(self, "discontinued")
            else None
        )
        _membership_id=(
            self.membership_id
            if hasattr(self, "membership_id")
            else None
        )
        _is_third_party_discount_pricing=(
            self.is_third_party_discount_pricing
            if hasattr(self, "is_third_party_discount_pricing")
            else None
        )
        _priority=(
            self.priority
            if hasattr(self, "priority")
            else None
        )
        _sell_at_location_ids=(
            self.sell_at_location_ids
            if hasattr(self, "sell_at_location_ids")
            else None
        )
        _use_at_location_ids=(
            self.use_at_location_ids
            if hasattr(self, "use_at_location_ids")
            else None
        )
        _expiration_unit=(
            self.expiration_unit
            if hasattr(self, "expiration_unit")
            else None
        )
        _expiration_length=(
            self.expiration_length
            if hasattr(self, "expiration_length")
            else None
        )
        _expiration_type=(
            self.expiration_type
            if hasattr(self, "expiration_type")
            else None
        )
        _restricted_membership_ids=(
            self.restricted_membership_ids
            if hasattr(self, "restricted_membership_ids")
            else None
        )
        return (
            f"{self.__class__.__name__}("
            f"product_id={_product_id!s}, "
            f"name={_name!s}, "
            f"price={_price!s}, "
            f"online_price={_online_price!s}, "
            f"count={_count!s}, "
            f"sell_online={_sell_online!s}, "
            f"revenue_category={_revenue_category!s}, "
            f"discontinued={_discontinued!s}, "
            f"membership_id={_membership_id!s}, "
            f"is_third_party_discount_pricing={_is_third_party_discount_pricing!s}, "
            f"priority={_priority!s}, "
            f"sell_at_location_ids={_sell_at_location_ids!s}, "
            f"use_at_location_ids={_use_at_location_ids!s}, "
            f"expiration_unit={_expiration_unit!s}, "
            f"expiration_length={_expiration_length!s}, "
            f"expiration_type={_expiration_type!s}, "
            f"restricted_membership_ids={_restricted_membership_ids!s}, "
            f")"
        )



"""mindbodypublicapi.

This file was automatically generated by APIMATIC v3.0 ( https://www.apimatic.io ).
"""

# ruff: noqa: E501
from mindbodypublicapi.models.client_document import (
    ClientDocument,
)


class UploadClientDocumentRequest(object):
    """Implementation of the 'UploadClientDocumentRequest' model.

    Attributes:
        client_id (str): The RSSID of the client for whom the document is to be
            uploaded.
        file (ClientDocument): The model property of type ClientDocument.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "client_id": "ClientId",
        "file": "File",
    }

    def __init__(
        self,
        client_id=None,
        file=None):
        """Initialize a UploadClientDocumentRequest instance."""
        # Initialize members of the class
        self.client_id = client_id
        self.file = file

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Create an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        client_id =\
            dictionary.get("ClientId")\
            if dictionary.get("ClientId")\
                else None
        file =\
            ClientDocument.from_dictionary(
                dictionary.get("File"))\
                if dictionary.get("File") else None

        # Return an object of this model
        return cls(client_id,
                   file)

    def __repr__(self):
        """Return a unambiguous string representation."""
        _client_id=self.client_id
        _file=self.file
        return (
            f"{self.__class__.__name__}("
            f"client_id={_client_id!r}, "
            f"file={_file!r}, "
            f")"
        )

    def __str__(self):
        """Return a human-readable string representation."""
        _client_id=self.client_id
        _file=self.file
        return (
            f"{self.__class__.__name__}("
            f"client_id={_client_id!s}, "
            f"file={_file!s}, "
            f")"
        )
